﻿using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.Events;

namespace MaskSlap
{
    public class Guard : MonoBehaviour
    {
        private enum State
        {
            Idle,
            ChasePlayer,
            PlayerCaptured
        }

        public UnityEvent OnCapturedPlayer { get; private set; }

        [SerializeField]
        private Sensor capturingSensor;
        [SerializeField]
        private Sensor innerSensor;
        [SerializeField]
        private Sensor outerSensor;

        private State state = State.Idle;
        private CharacterController controller;
        private Vector3 targetPosition;
        [SerializeField]
        private float runAwaySpeed = 3f;
        private EnemyAnimationHandler enemyAnimationHandler;
        [SerializeField]
        private float gettingCaughtTimeThreshold = 2;

        [SerializeField]
        private GameObject alertIcon;
        [SerializeField]
        private GameObject gettingCaughtObject;
        [SerializeField]
        private SpriteRenderer gettingCaught;

        private bool isPlayerInRangeOfGettingCaught;
        private float playerInRangeTicker;

        private void Awake()
        {
            InitSensors();
            controller = GetComponent<CharacterController>();
            targetPosition = transform.position;
            enemyAnimationHandler = GetComponent<EnemyAnimationHandler>();
        }

        #region Behaviour
        private void Update()
        {
            switch (state)
            {
                case State.Idle:
                    UpdateIdle();
                    break;
                case State.ChasePlayer:
                    UpdateChasePlayer();
                    break;
                case State.PlayerCaptured:
                    UpdatePlayerCaptured();
                    break;
            }

            if (isPlayerInRangeOfGettingCaught)
            {
                playerInRangeTicker -= Time.deltaTime;
                var t = playerInRangeTicker / gettingCaughtTimeThreshold;
                gettingCaught.material.SetFloat("_Fill", 1 - t);
                if (playerInRangeTicker <= 0)
                {
                    state = State.PlayerCaptured;
                }
            }
        }

        private void UpdateIdle()
        {
            enemyAnimationHandler.PlayIdle();
        }

        private void UpdateChasePlayer()
        {
            if (transform.position != targetPosition)
            {
                var delta = targetPosition - transform.position;
                var direction = delta.normalized;
                var toMove = (runAwaySpeed * Time.deltaTime) * direction;
                if (toMove.sqrMagnitude > delta.sqrMagnitude)
                {
                    toMove = delta;
                }
                controller.Move(toMove);
                enemyAnimationHandler.PlayRunning();
                if (Mathf.Approximately(toMove.sqrMagnitude, 0f))
                {
                    state = State.Idle;
                }
            }
        }

        private void UpdatePlayerCaptured()
        {
            enemyAnimationHandler.PlayGettingHit();
            PlayerAnimationHandler._instance.PlayGettingCaught();
            GameState.instance.PlayerCaptured();
        }

        private void SetChasePosition(Collider other)
        {
            targetPosition = other.transform.position;
            targetPosition.y = transform.position.y;
            state = State.ChasePlayer;
        }
        public void GameOver()
        {
            enabled = false;
        }
        #endregion

        #region sensors
        private void InitSensors()
        {
            innerSensor.OnSensorEnter.AddListener(OnInnerSensorEnter);
            innerSensor.OnSensorStay.AddListener(OnInnerSensorStay);
            innerSensor.OnSensorExit.AddListener(OnInnerSensorExit);

            outerSensor.OnSensorEnter.AddListener(OnOuterSensorEnter);
            outerSensor.OnSensorExit.AddListener(OnOuterSensorExit);

            capturingSensor.OnSensorEnter.AddListener(OnCaptureSensorEnter);
            capturingSensor.OnSensorExit.AddListener(OnCaptureSensorExit);
        }

        private void OnInnerSensorEnter(Collider other)
        {
          
        }
        private void OnInnerSensorStay(Collider other)
        {
            SetChasePosition(other);
        }
        private void OnInnerSensorExit(Collider other)
        {
        }

        private void OnOuterSensorEnter(Collider other)
        {
            alertIcon.SetActive(true);
        }
        private void OnOuterSensorExit(Collider other)
        {
            alertIcon.SetActive(false);
        }

        private void OnCaptureSensorEnter(Collider other)
        {
            playerInRangeTicker = gettingCaughtTimeThreshold;
            isPlayerInRangeOfGettingCaught = true;
            gettingCaughtObject.gameObject.SetActive(true);
            alertIcon.SetActive(false);
        }
        private void OnCaptureSensorExit(Collider other)
        {
            gettingCaughtObject.gameObject.SetActive(false);
            isPlayerInRangeOfGettingCaught = false;
            alertIcon.SetActive(true);
        }
        #endregion

#if UNITY_EDITOR
        private void OnDrawGizmosSelected()
        {
            if (targetPosition != transform.position)
            {
                var color = Gizmos.color;

                Gizmos.color = Color.red;
                Gizmos.DrawLine(transform.position, targetPosition);

                Gizmos.color = color;
            }
        }
#endif
    }
}
