﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

namespace MaskSlap
{
    public class Player : MonoBehaviour
    {
        public static Player _player { get; private set; }

        private bool isControllable = false;

        private float currentSpeed = 5;
        private bool isDashing;
        private Vector2 moveDirectionAtTheTimeOfDashing;

        [SerializeField]
        private float normalSpeed = 5;

        [Header("Dash")]
        [SerializeField]
        private float dashDuration = 1;
        [SerializeField]
        private float dashSpeedBoost = 2;
        [SerializeField]
        private float dashCooldownTimer;

        [Header("Attack")]
        [SerializeField]
        private float attackDetectionRadius = 2;
        [SerializeField]
        private float attackCooldownTimer;
        [SerializeField]
        private LayerMask slapLayer;

        [Header("Feedback")]
        [SerializeField]
        private SpriteRenderer dashFeedback;
        [SerializeField]
        private SpriteRenderer attackFeedback;

        private CharacterController charController;

        private Vector3 defaultDashFeedbackSize, defaultAttackFeedbackSize;

        private bool isMoving;
        private bool isDashOnCooldown, isAttackOnCooldown;
        private Coroutine dashCooldownCoroutine, dashEffectCoroutine, attackCooldownCoroutine;

        private void Awake()
        {
            _player = this;
            charController = GetComponent<CharacterController>();
            defaultDashFeedbackSize = dashFeedback.transform.localScale;
            defaultAttackFeedbackSize = attackFeedback.transform.localScale;
        }

        private void Start()
        {
            RegisterToInputCallbackEvents();
        }

        private void RegisterToInputCallbackEvents()
        {
            InputHandler._instance.OnDashEvent -= OnDashInputCallback;
            InputHandler._instance.OnDashEvent += OnDashInputCallback;

            InputHandler._instance.OnAttackEvent -= OnAttackInputCallback;
            InputHandler._instance.OnAttackEvent += OnAttackInputCallback;
        }

        private void Update()
        {
            if (isControllable)
            {
                Vector3 movementDir = isDashing ? moveDirectionAtTheTimeOfDashing : InputHandler._instance.GetMovementAxis();
                movementDir = new Vector3(movementDir.x, 0, movementDir.y);
                charController.SimpleMove(movementDir * currentSpeed);
                if (isMoving && movementDir == Vector3.zero)
                {
                    //Stopped
                    isMoving = false;
                    AudioHandler._instance.PlayFootsteps(false);
                    PlayerAnimationHandler._instance.PlayMovementAnimations(movementDir);
                }
                else if (!isMoving && movementDir != Vector3.zero)
                {
                    //Started moving
                    isMoving = true;
                    AudioHandler._instance.PlayFootsteps(true);
                }
                if (isMoving)
                {
                    PlayerAnimationHandler._instance.PlayMovementAnimations(movementDir);
                }
            }
        }

        public void StartPlaying()
        {
            GameObject spawnPosition = GameObject.FindWithTag("SpawnPosition");
            transform.position = spawnPosition.transform.position;

            Debug.Log(spawnPosition.name, spawnPosition.transform);

            Destroy(spawnPosition);

            isControllable = true;
            AudioHandler._instance.PlayFootsteps(false);
            isMoving = false;
            currentSpeed = normalSpeed;
            isDashOnCooldown = false;
            isAttackOnCooldown = false;
        }

        public void GameWon()
        {
            isControllable = false;
        }

        public void GotCaptured()
        {
            isControllable = false;
        }

        private void OnDashInputCallback()
        {
            if (!isControllable)
            {
                return;
            }

            if (isDashing || isDashOnCooldown)
            {
                return;
            }

            //No dash when not moving at all.
            if (InputHandler._instance.GetMovementAxis() == Vector2.zero)
            {
                return;
            }

            if (dashEffectCoroutine != null)
            {
                StopCoroutine(dashEffectCoroutine);
            }
            dashEffectCoroutine = StartCoroutine(DashEffectTicker());

            if(dashCooldownCoroutine != null)
            {
                StopCoroutine(dashCooldownCoroutine);    
            }
            PlayerAnimationHandler._instance.PlayerDashAnimation();
            dashCooldownCoroutine = StartCoroutine(DashCooldownTicker());
        }

        private IEnumerator DashEffectTicker()
        {
            isDashing = true;
            AudioHandler._instance.PlaySFX(AUDIO_SFX_TYPE.DASH);
            moveDirectionAtTheTimeOfDashing = InputHandler._instance.GetMovementAxis();
            isDashOnCooldown = false;
            vfxmanager._instance.ShowDash();
            currentSpeed = normalSpeed + dashSpeedBoost;

            yield return new WaitForSeconds(dashDuration);

            isDashing = false;
            vfxmanager._instance.HideDash();
            currentSpeed = normalSpeed;
        }

        private IEnumerator DashCooldownTicker()
        {
            isDashOnCooldown = true;
            dashFeedback.color = Color.black;
            dashFeedback.transform.localScale = defaultDashFeedbackSize * 0.5f;
            yield return new WaitForSeconds(dashCooldownTimer);
            dashFeedback.transform.localScale = defaultDashFeedbackSize;
            dashFeedback.color = Color.white;
            isDashOnCooldown = false;
        }

        private IEnumerator AttackCooldownTicker()
        {
            isAttackOnCooldown = true;
            attackFeedback.color = Color.black;
            attackFeedback.transform.localScale = defaultAttackFeedbackSize * 0.5f;
            yield return new WaitForSeconds(attackCooldownTimer);
            attackFeedback.transform.localScale = defaultAttackFeedbackSize;
            attackFeedback.color = Color.white;
            isAttackOnCooldown = false;
        }

        private void OnAttackInputCallback()
        {
            if (!isControllable)
            {
                return;
            }

            if (isAttackOnCooldown)
            {
                return;
            }

            Politician tempPolitician = GetPoliticianInRange();
            PlayerAnimationHandler._instance.PlayerHitAnimation();
            if (tempPolitician != null)
            {
                AudioHandler._instance.PlaySFX(AUDIO_SFX_TYPE.ATTACK);
                tempPolitician.OnHit?.Invoke(tempPolitician);
                CameraController._camera.CameraShake();
            }
            else
            {
                AudioHandler._instance.PlaySFX(AUDIO_SFX_TYPE.ATTACKMISS);
            }

            if (attackCooldownCoroutine != null)
            {
                StopCoroutine(attackCooldownCoroutine);
            }
            attackCooldownCoroutine = StartCoroutine(AttackCooldownTicker());
        }

        private Politician GetPoliticianInRange()
        {
            Politician tempPolitician = null;
            Collider[] politicianColliders = Physics.OverlapSphere(transform.position, attackDetectionRadius, slapLayer);
            if (politicianColliders.Length > 0)
            {
                float minDist = Mathf.Infinity;
                for (int i = 0; i < politicianColliders.Length; i++)
                {
                    if (politicianColliders[i].CompareTag("Politician"))
                    {
                        float dist = Vector3.Distance(transform.position, politicianColliders[i].transform.position);
                        if (dist < minDist)
                        {
                            minDist = dist;
                            tempPolitician = politicianColliders[i].transform.GetComponent<Politician>();
                        }
                    }
                }
            }

            return tempPolitician;
        }

#if UNITY_EDITOR
        private void OnDrawGizmos()
        {
            var color = Gizmos.color;

            Gizmos.color = Color.white;
            Gizmos.DrawWireSphere(transform.position, attackDetectionRadius);

            Gizmos.color = color;
        }
#endif
    }
}
