﻿using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.Events;

namespace MaskSlap
{
    public class Politician : MonoBehaviour
    {
        private enum State
        {
            Idle,
            RunAway,
            GotHit
        }

        public class PoliticianHitEvent : UnityEvent<Politician> { }

        public PoliticianHitEvent OnHit { get; private set; } = new PoliticianHitEvent();

        [SerializeField]
        private Sensor gettingSlappedSensor;
        [SerializeField]
        private Sensor innerSensor;
        [SerializeField]
        private Sensor outerSensor;

        private State state = State.Idle;
        private CharacterController controller;
        private Vector3 targetPosition;
        [SerializeField]
        private float runAwayToDistance = 8f;
        [SerializeField]
        private float runAwaySpeed = 3f;
        [SerializeField]
        private LayerMask environmentMask;
        private int environmentMaskVal;
        private EnemyAnimationHandler enemyAnimationHandler;

        [SerializeField]
        private GameObject alertIcon;
        [SerializeField]
        private GameObject inRangeFeedback;

        private void Awake()
        {
            InitSensors();
            controller = GetComponent<CharacterController>();
            targetPosition = transform.position;
            environmentMaskVal = environmentMask.value;
            enemyAnimationHandler = GetComponent<EnemyAnimationHandler>();
        }

        #region Behaviour
        private void Update()
        {
            switch (state)
            {
                case State.Idle:
                    UpdateIdle();
                    break;
                case State.RunAway:
                    UpdateRunAway();
                    break;
                case State.GotHit:
                    UpdateGotHit();
                    break;
            }
        }

        private void UpdateIdle()
        {
            enemyAnimationHandler.PlayIdle();
        }

        private void UpdateRunAway()
        {
            if (transform.position != targetPosition)
            {
                var delta = targetPosition - transform.position;
                var direction = delta.normalized;
                var toMove = (runAwaySpeed * Time.deltaTime) * direction;
                if (toMove.sqrMagnitude > delta.sqrMagnitude)
                {
                    toMove = delta;
                }
                controller.Move(toMove);
                enemyAnimationHandler.PlayRunning();
                if (Mathf.Approximately(toMove.sqrMagnitude, 0f))
                {
                    state = State.Idle;
                }
            }
            else
            {
                state = State.Idle;
            }
        }

        private void UpdateGotHit()
        {
        }

        int depth = 0;
        private void StartRunningAway(Vector3 otherPos)
        {
            var delta = transform.position - otherPos;
            delta.y = 0f;
            delta.Normalize();
            if (Physics.Raycast(transform.position, delta, out var hitInfo, runAwayToDistance, environmentMaskVal))
            {
                targetPosition = hitInfo.point + (2.4f * controller.radius * hitInfo.normal);
            }
            else
            {
                targetPosition = transform.position + (runAwayToDistance * delta);
            }
            if ((targetPosition - transform.position).sqrMagnitude < 2f && depth++ < 2)
            {
                StartRunningAway(targetPosition);
                return;
            }
            state = State.RunAway;
        }

        public void GotSlapped()
        {
            enemyAnimationHandler.PlayGettingHit();

            enabled = false;
            controller.enabled = false;
            innerSensor.gameObject.SetActive(false);
            outerSensor.gameObject.SetActive(false);
            gettingSlappedSensor.gameObject.SetActive(false);
            alertIcon.SetActive(false);
            inRangeFeedback.SetActive(false);
            state = State.GotHit;

            StartCoroutine(SlowDownOnHit());
        }

        [SerializeField]
        private float timeSlowDownDuration = 0.5f;
        [SerializeField]
        private float timeSlowDownOnHit = 0.05f;
        private IEnumerator SlowDownOnHit()
        {
            Time.timeScale = timeSlowDownOnHit;
            yield return new WaitForSecondsRealtime(timeSlowDownDuration);
            Time.timeScale = 1f;
        }

        public void GameOver()
        {
            enabled = false;
        }
        #endregion

        #region sensors
        private void InitSensors()
        {
            innerSensor.OnSensorEnter.AddListener(OnInnerSensorEnter);
            innerSensor.OnSensorStay.AddListener(OnInnerSensorStay);
            innerSensor.OnSensorExit.AddListener(OnInnerSensorExit);

            outerSensor.OnSensorEnter.AddListener(OnOuterSensorEnter);
            outerSensor.OnSensorExit.AddListener(OnOuterSensorExit);

            gettingSlappedSensor.OnSensorEnter.AddListener(OnSlapRangeSensorEnter);
            gettingSlappedSensor.OnSensorExit.AddListener(OnSlapRangeSensorExit);
        }

        private void OnSlapRangeSensorEnter(Collider other)
        {
            inRangeFeedback.SetActive(true);
        }

        private void OnSlapRangeSensorExit(Collider other)
        {
            inRangeFeedback.SetActive(false);
        }

        private void OnInnerSensorEnter(Collider other)
        {

        }
        private void OnInnerSensorStay(Collider other)
        {
            if (state == State.Idle)
            {
                depth = 0;
                StartRunningAway(other.transform.position);
            }
        }
        private void OnInnerSensorExit(Collider other)
        {
        }

        private void OnOuterSensorEnter(Collider other)
        {
            alertIcon.SetActive(true);
        }
        private void OnOuterSensorExit(Collider other)
        {
            alertIcon.SetActive(false);
        }
        #endregion

#if UNITY_EDITOR
        private void OnDrawGizmosSelected()
        {
            if (targetPosition != transform.position)
            {
                var color = Gizmos.color;

                Gizmos.color = Color.red;
                Gizmos.DrawLine(transform.position, targetPosition);

                Gizmos.color = color;
            }
        }
#endif
    }
}
